import datetime
from typing import ClassVar, List, NamedTuple, Optional, Union
from typing_extensions import Literal

from .asset import Asset
from .channel import StoreChannel, TextChannel, VoiceChannel
from .enums import ChannelType, VerificationLevel
from .guild import Guild
from .mixins import Hashable
from .object import Object
from .user import User

_VALID_ICON_FORMATS = Literal['jpeg', 'jpg', 'webp', 'png']

class PartialInviteChannel(NamedTuple):
    id: int
    name: str
    type: ChannelType
    @property
    def mention(self) -> str: ...
    @property
    def created_at(self) -> datetime.datetime: ...

class PartialInviteGuild:
    features: List[str]
    icon: Optional[str]
    banner: Optional[str]
    id: int
    name: str
    splash: Optional[str]
    verification_level: VerificationLevel
    description: Optional[str]
    @property
    def created_at(self) -> datetime.datetime: ...
    @property
    def icon_url(self) -> Asset: ...
    def icon_url_as(
        self, *, format: Optional[_VALID_ICON_FORMATS] = ..., size: int = ...,
    ) -> Asset: ...
    @property
    def banner_url(self) -> Asset: ...
    def banner_url_as(
        self, *, format: _VALID_ICON_FORMATS = ..., size: int = ...
    ) -> Asset: ...
    @property
    def splash_url(self) -> Asset: ...
    def splash_url_as(
        self, *, format: _VALID_ICON_FORMATS = ..., size: int = ...
    ) -> Asset: ...

class Invite(Hashable):
    BASE: ClassVar[str]

    max_age: int
    code: str
    guild: Optional[Union[Guild, Object, PartialInviteGuild]]
    revoked: bool
    created_at: datetime.datetime
    temporary: bool
    uses: int
    max_uses: int
    inviter: User
    approximate_member_count: Optional[int]
    approximate_presence_count: Optional[int]
    channel: Union[
        TextChannel, VoiceChannel, StoreChannel, Object, PartialInviteChannel
    ]
    def __hash__(self) -> int: ...
    @property
    def id(self) -> str: ...
    @property
    def url(self) -> str: ...
    async def delete(self, *, reason: Optional[str] = ...) -> None: ...



from typing import Any, List

import discord

from .context import Context

class Converter:
    async def convert(self, ctx: Context, argument: str) -> Any: ...

class IDConverter(Converter): ...

class MemberConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.Member: ...

class UserConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.User: ...

class MessageConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.Message: ...

class TextChannelConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.TextChannel: ...

class VoiceChannelConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.VoiceChannel: ...

class CategoryChannelConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.CategoryChannel: ...

class ColourConverter(Converter):
    async def convert(self, ctx: Context, argument: str) -> discord.Colour: ...

class RoleConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.Role: ...

class GameConverter(Converter):
    async def convert(self, ctx: Context, argument: str) -> discord.Game: ...

class InviteConverter(Converter):
    async def convert(self, ctx: Context, argument: str) -> discord.Invite: ...

class EmojiConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.Emoji: ...

class PartialEmojiConverter(Converter):
    async def convert(self, ctx: Context, argument: str) -> discord.PartialEmoji: ...

class clean_content(Converter):
    def __init__(
        self,
        *,
        fix_channel_mentions: bool = ...,
        use_nicknames: bool = ...,
        escape_markdown: bool = ...,
    ) -> None: ...
    async def convert(self, ctx: Context, argument: str) -> str: ...

Greedy = List



import asyncio
from typing import Callable, ClassVar, Optional, Tuple

from .abc import Connectable
from .channel import VoiceChannel
from .guild import Guild
from .player import AudioSource
from .user import ClientUser

class VoiceClient:
    warn_nacl: ClassVar[bool] = ...
    supported_modes: ClassVar[Tuple[str, ...]] = ...

    session_id: str
    token: str
    endpoint: str
    channel: Connectable
    loop: asyncio.AbstractEventLoop
    source: Optional[AudioSource]
    @property
    def guild(self) -> Optional[Guild]: ...
    @property
    def user(self) -> ClientUser: ...
    def checked_add(self, attr: str, value: int, limit: int) -> None: ...
    async def start_handshake(self) -> None: ...
    async def terminate_handshake(self, *, remove: bool = ...) -> None: ...
    async def connect(
        self, *, reconnect: bool = ..., _tries: int = ..., do_handshake: bool = ...
    ) -> None: ...
    async def poll_voice_ws(self, reconnect: bool) -> None: ...
    async def disconnect(self, *, force: bool = ...) -> None: ...
    async def move_to(self, channel: VoiceChannel) -> None: ...
    def is_connected(self) -> bool: ...
    def play(
        self,
        source: AudioSource,
        *,
        after: Optional[Callable[[Optional[Exception]], None]] = ...,
    ) -> None: ...
    def is_playing(self) -> bool: ...
    def is_paused(self) -> bool: ...
    def stop(self) -> None: ...
    def pause(self) -> None: ...
    def resume(self) -> None: ...
    def send_audio_packet(self, data: bytes, *, encode: bool = ...) -> None: ...



import asyncio
from typing import Any, List, Optional, Tuple, Union, overload

import aiohttp

from .activity import BaseActivity
from .client import Client
from .enums import Status
from .guild import Guild

class Shard:
    @property
    def id(self) -> int: ...
    def is_pending(self) -> bool: ...
    def complete_pending_reads(self) -> None: ...
    def launch_pending_reads(self) -> None: ...
    def wait(self) -> Optional[asyncio.Future[None]]: ...
    async def poll(self) -> None: ...
    def get_future(self) -> asyncio.Future[None]: ...

class AutoShardedClient(Client):
    shard_ids: Optional[Union[List[int], Tuple[int]]]
    @overload
    def __init__(
        self,
        *args: Any,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        shard_ids: Union[List[int], Tuple[int]] = ...,
        shard_count: int = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        max_messages: Optional[int] = ...,
        fetch_offline_members: bool = ...,
        status: Optional[Status] = ...,
        activity: Optional[BaseActivity] = ...,
        heartbeat_timeout: float = ...,
        **kwargs: Any,
    ) -> None: ...
    @overload
    def __init__(
        self,
        *args: Any,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        shard_count: Optional[int] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        max_messages: Optional[int] = ...,
        fetch_offline_members: bool = ...,
        status: Optional[Status] = ...,
        activity: Optional[BaseActivity] = ...,
        heartbeat_timeout: float = ...,
        **kwargs: Any,
    ) -> None: ...
    @property
    def latency(self) -> float: ...
    @property
    def latencies(self) -> List[Tuple[int, float]]: ...
    async def request_offline_members(self, *guilds: Guild) -> None: ...
    async def launch_shard(self, gateway: str, shard_id: int) -> None: ...
    async def launch_shards(self) -> None: ...
    async def close(self) -> None: ...
    async def change_presence(
        self,
        *,
        activity: Optional[BaseActivity] = ...,
        status: Optional[Status] = ...,
        afk: bool = ...,
        shard_id: Optional[int] = ...,
    ) -> None: ...



import sys
from typing import Any, BinaryIO, ClassVar, FrozenSet, Union
from typing_extensions import Final

if sys.version_info >= (3, 6):
    from os import PathLike

VALID_STATIC_FORMATS: Final[FrozenSet[str]] = ...
VALID_AVATAR_FORMATS: Final[FrozenSet[str]] = ...

class Asset:
    BASE: ClassVar[str]
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    async def read(self) -> bytes: ...
    if sys.version_info >= (3, 6):
        async def save(
            self, fp: Union[BinaryIO, PathLike[str], str], *, seek_begin: bool = ...
        ) -> int: ...
    else:
        async def save(
            self, fp: Union[BinaryIO, str], *, seek_begin: bool = ...
        ) -> int: ...



from typing import Any, Callable, ClassVar, Dict, Iterator, Tuple, TypeVar

_F = TypeVar('_F', bound=flag_value)

class flag_value:
    flag: int
    def __init__(self, func: Callable[[Any], int]) -> None: ...
    def __get__(self, instance: Any, owner: Any) -> bool: ...
    def __set__(self, instance: Any, value: bool) -> None: ...

class BaseFlags:
    value: int = ...
    def __init__(self, **kwargs: bool) -> None: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    def __iter__(self) -> Iterator[Tuple[str, bool]]: ...

class SystemChannelFlags(BaseFlags):
    VALID_FLAGS: ClassVar[Dict[str, int]]

    join_notifications: flag_value
    premium_subscriptions: flag_value

class MessageFlags(BaseFlags):
    VALID_FLAGS: ClassVar[Dict[str, int]]

    crossposted: flag_value
    is_crossposted: flag_value
    suppress_embeds: flag_value
    source_message_deleted: flag_value
    urgent: flag_value



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['discord-stubs']

package_data = \
{'': ['*'], 'discord-stubs': ['ext/*', 'ext/commands/*', 'ext/tasks/*']}

install_requires = \
['mypy>=0.782,<0.783', 'typing-extensions>=3.7.4,<4.0.0']

setup_kwargs = {
    'name': 'discord.py-stubs',
    'version': '1.3.4.0',
    'description': 'discord.py stubs',
    'long_description': None,
    'author': 'Bryan Forbes',
    'author_email': 'bryan@reigndropsfall.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
